@@ -39,9 +39,11 @@
   
   #:export
   (&gw-bad-typespec
+   raise-bad-typespec
    
    <gw-item>
    description
+   all-types-referenced
    
    <gw-constant>
    value typespec 
@@ -52,7 +54,6 @@
    arguments argument-types
    return-type return-typespec
    generic-name 
-   all-types-referenced
    
    <gw-type>
    class-name
@@ -89,7 +90,8 @@
    <gw-wrapset>
    name language wrapsets-depended-on
    fold-types for-each-type lookup-type fold-functions
-
+   consider-types?
+   
    add-item! add-type! add-constant! add-function!
    add-client-item!
    
@@ -146,6 +148,9 @@
                #:init-keyword #:description
                #:init-value #f))
 
+(define-method (all-types-referenced (sel <gw-item>))
+  '())
+
 ;; Upgrade the GOOPS class-name procedure
 ;; barfs with GUILE CVS: class-name not defined -- jcn
 (if (defined? 'class-name)
@@ -183,6 +188,25 @@
                              (symbol->string
                               (name type))) "_" suffix)))
 
+;; Here because needs <gw-type>
+(define-method (raise-bad-typespec type (options <list>) (msg <string>) . args)
+  (raise (condition
+          (&gw-bad-typespec
+           (type type) (options options)
+           (message (apply format #f msg args))))))
+
+(define-method (raise-bad-typespec spec (msg <string>) . args)
+  (raise (condition
+          (&gw-bad-typespec
+           (spec spec)
+           (message (apply format #f msg args))))))
+
+(define-method (raise-stacked (next &condition) (msg <string>) . args)
+  (raise (condition
+          (&gw-stacked
+           (next next)
+           (message (apply format #f msg args))))))
+  
 ;;;
 ;;; Values
 ;;;
@@ -281,7 +305,7 @@
   (list (type typespec)))
 
 (define-method (add-option! (self <gw-typespec>) (option <symbol>))
-  (slot-set! self 'options (cons option (slot-ref self 'options))))
+  (slot-push! self 'options option))
 
 (define-method (make-typespec (type <gw-type>) (options <list>))
   (check-typespec-options type options)
@@ -391,6 +415,9 @@
 (define-method (type (constant <gw-constant>))
   (type (typespec constant)))
 
+(define-method (all-types-referenced (self <gw-constant>))
+  (all-types (typespec self)))
+
 ;;;
 ;;; Code (currently nested string lists)
 ;;;
@@ -399,6 +426,9 @@
 (define-method (render (code <list>) (port <port>))
   (flatten-display code port))
 
+(define-method (render (code <string>) (port <port>))
+  (render (list code) port))
+
 (define-method (no-op? (code <list>)) (null? list))
 
 
@@ -411,7 +441,7 @@
 
 (define-method (initialize (class <gw-wrapset-class>) initargs)
   (next-method)
-  
+
   (let-keywords
    initargs #t (language id types (dependencies '()))
    (if (not language)
@@ -428,7 +458,7 @@
    (if id
        (class-slot-set! class 'name id))
 
-   (class-slot-set-supers-union! class 'dependency-ids <object> dependencies)
+   (class-slot-set-supers-union! class 'dependency-ids dependencies)
    
    (class-slot-set! class 'type-classes (make-hash-table 7))
    (cond
@@ -473,28 +503,28 @@
                   (slot-ref self 'dependency-ids))))
 
 (define-method (add-item! (self <gw-wrapset>) (item <gw-item>))
-  (slot-set! self 'items (cons item (slot-ref self 'items))))
+  (slot-push! self 'items item))
 
 (define-method (add-client-item! (self <gw-wrapset>) (item <gw-item>))
-  (slot-set! self 'client-items (cons item (slot-ref self 'client-items))))
+  (slot-push! self 'client-items item))
 
 (define-method (add-type! (ws <gw-wrapset>) (type <gw-type>))
-  (slot-set! ws 'types (cons type (slot-ref ws 'types)))
-  (slot-set! ws 'items (cons type (slot-ref ws 'items)))
+  (slot-push! ws 'types type)
+  (slot-push! ws 'items type)
   (if (hashq-ref (slot-ref ws 'type-hash) (name type))
       (error "trying to double-register type ~S in ~S" type ws))
   (hashq-set! (slot-ref ws 'type-hash) (name type) type))
 
 (define-method (add-function! (ws <gw-wrapset>) (function <gw-function>))
-  (slot-set! ws 'items (cons function (slot-ref ws 'items)))
-  (slot-set! ws 'functions (cons function (slot-ref ws 'functions)))
+  (slot-push! ws 'items function)
+  (slot-push! ws 'functions function)
   (if (generic-name function)
       (let ((handle (hashq-create-handle! (slot-ref ws 'generic-hash)
                                           (generic-name function) '())))
         (set-cdr! handle (cons function (cdr handle))))))
 
 (define-method (add-constant! (ws <gw-wrapset>) (constant <gw-constant>))
-  (slot-set! ws 'items (cons constant (slot-ref ws 'items))))
+  (slot-push! ws 'items constant))
 
 (define-method (fold-types kons knil (ws <gw-wrapset>))
   (fold-right kons knil (slot-ref ws 'types)))
@@ -525,6 +555,9 @@
 (define-method (for-each-function proc (ws <gw-wrapset>))
   (for-each proc (reverse (slot-ref ws 'functions))))
 
+(define-method (consider-types? (wrapset <gw-wrapset>) (item <gw-item>))
+  #t)
+
 (define (resolve-typespec wrapset spec)
   (let* ((form (cond
                 ((symbol? spec) (list spec))
@@ -544,56 +577,36 @@
                  ((symbol? elt)
                   elt)
                  (else
-                  (raise (condition
-                          (&gw-bad-typespec
-                           (type type) (options (cdr form))
-                           (message (format #f "bad option ~S" elt))))))))
+                  (raise-bad-typespec type (cdr form) "bad option ~S" elt))))
               (cdr form)))
-        (raise (condition
-                (&gw-bad-typespec
-                 (type (car form)) (options (cdr form))
-                 (message (format #f "no such type in wrapset `~S'"
-                                  (name wrapset)))))))))
+        (raise-bad-typespec (car form) (cdr form)
+                            "no such type in wrapset `~S'"
+                            (name wrapset)))))
 
 (define (resolve-arguments wrapset argspecs)
   (define (argument i spec)
     (if (not (and (list? spec) (>= (length spec) 2)))
-        (raise (condition
-                (&gw-bad-typespec
-                 (spec spec)
-                 (message
-                  "argument spec must be a (at least) two-element list")))))
+        (raise-bad-typespec
+         spec
+         "argument spec must be a (at least) two-element list"))
     (let ((ts (car spec)))
       (guard
        (c
-        (#t (raise (condition
-                    (&gw-stacked
-                     (next c)
-                     (message (format #f "while processing argument `~S'"
-                                      (second spec))))))))
-       (apply make <gw-argument>
-              #:number i
-              #:name (second spec)
-              #:typespec (resolve-typespec wrapset ts)
-              (fold
-               (lambda (opt rest)
-                 (if (not (and (list? opt) (= (length opt) 2)))
-                     (raise
-                      (condition
-                       (&gw-bad-typespec
-                        (spec spec)
-                        (message
-                         (format #f "invalid argument option ~S" opt))))))
-                 (case (first opt)
-                   ((default) (cons #:default (cons (second opt) rest)))
-                   (else
-                    (raise
-                     (condition
-                      (&gw-bad-typespec
-                       (spec spec)
-                       (message
-                        (format #f "unknown argument option ~S" opt))))))))
-               '() (cddr spec))))))
+        (#t (raise-stacked c "while processing argument `~S'" (second spec))))
+       (apply
+        make <gw-argument>
+        #:number i
+        #:name (second spec)
+        #:typespec (resolve-typespec wrapset ts)
+        (fold
+         (lambda (opt rest)
+           (if (not (and (list? opt) (= (length opt) 2)))
+               (raise-bad-typespec spec "invalid argument option ~S" opt))
+           (case (first opt)
+             ((default) (cons #:default (cons (second opt) rest)))
+             (else
+              (raise-bad-typespec spec "unknown argument option ~S" opt))))
+         '() (cddr spec))))))
   
   (let loop ((i 0) (specs argspecs) (args '()))
     (if (null? specs)
@@ -616,11 +629,7 @@
    args #f (name returns c-name arguments description generic-name)
    (guard
     (c
-     (#t (raise
-          (condition
-           (&gw-stacked
-            (next c)
-            (message (format #f "while processing function `~S'" name)))))
+     (#t (raise-stacked c "while processing function `~S'" name)
          ;; TODO: Find a way how go on and exit with failure at the end
          (exit 1)))
     
@@ -643,7 +652,9 @@
                             #:description description))))
 
 (define-method (all-types-referenced (self <gw-wrapset>))
-  (append-map all-types-referenced (slot-ref self 'functions)))
+  (append-map all-types-referenced
+              (filter (lambda (item) (consider-types? self item))
+                      (slot-ref self 'items))))
 
 
 ;;;
@@ -680,7 +691,7 @@
          (wrapset-name (name wrapset))
          (wrapset-name-c-sym (any-str->c-sym-str
                               (symbol->string wrapset-name)))
-         (wrapset-init-func (string-append "gw_init_wrapset_"
+         (wrapset-init-func (string-append "gw_initialize_wrapset_"
                                            wrapset-name-c-sym)))
 
     (define (output-initializer-cg cg)
@@ -711,6 +722,7 @@
 
     (display "}\n" port)))
 
+;; Entry main point for wrapset generation
 (define-method (generate-wrapset (lang <symbol>)
                                  (name <symbol>)
                                  (basename <string>))
@@ -783,7 +795,8 @@
     (render-client-types client-global-declarations-cg)
 
     (dsp-list
-     (list "void gw_init_wrapset_" wrapset-name-c-sym "(GWLangArena);\n"))
+     (list
+      "void gw_initialize_wrapset_" wrapset-name-c-sym "(GWLangArena);\n"))
     
     (render (global-definitions-cg wrapset) port)
     (render-items global-definitions-cg)
@@ -796,7 +809,7 @@
     (dsp-list
      (list
       "void\n"
-      "gw_init_wrapset_" wrapset-name-c-sym "(GWLangArena gw__arena) {\n"
+      "gw_initialize_wrapset_" wrapset-name-c-sym "(GWLangArena gw__arena) {\n"
       "  static int gw_wrapset_initialized = 0;\n"
       "\n"))
 
@@ -1043,3 +1056,6 @@
   (expand-helper tree param allowed-errors tree))
 
 
+;; Hook in compat layer
+(module-use! (resolve-interface '(g-wrap))
+             (resolve-interface '(g-wrap compat)))
