@@ -57,12 +57,8 @@
                  #:init-keyword #:function-rti?
                  #:init-value #t))
 
-(define-method (all-types-referenced (self <gw-wrapset>))
-  (append-map (lambda (func)
-                (if (uses-rti-for-function? self func)
-                    '()
-                    (all-types-referenced func)))
-              (slot-ref self 'functions)))
+(define-method (consider-types? (self <gw-rti-wrapset>) (func <gw-function>))
+  (not (uses-rti-for-function? self func)))
 
 (define-class <gw-rti-value> (<gw-value>))
 
@@ -92,7 +88,7 @@
   (let-keywords
    initargs #t ((allowed-options '()))
    (class-slot-set-supers-union!
-    class 'allowed-options <gw-type> allowed-options)))
+    class 'allowed-options allowed-options)))
 
 (define-class <gw-rti-type> (<gw-type>)
   (allowed-options #:init-value '() #:allocation #:each-subclass)
