@@ -37,7 +37,7 @@
   (&gw-bad-element
    element tree
 
-   guard/handle call-with-output-file/cleanup
+   call-with-output-file/cleanup
    
    <gw-cs-labels>
    goto-cg label-cg
@@ -45,7 +45,8 @@
    flatten-display flatten-string separate-by any-str->c-sym-str
    gen-c-tmp str-translate
 
-   class-slot-set-supers-union!))
+   class-slot-set-supers-union!)
+  #:export-syntax (guard/handle slot-push!))
 
 ;;; Condition stuff
 
@@ -97,6 +98,9 @@
           (false-if-exception
            (delete-file (string-append file-name "~")))))))
 
+(define-macro (slot-push! obj slot value)
+  `(slot-set! ,obj ,slot (cons ,value (slot-ref ,obj ,slot))))
+
 ;;; Support for C labels - declare them only when needed
 
 (define-class <gw-cs-labels> ()
@@ -220,7 +224,7 @@
 		 (loop (+ 1 i)))))))))
 
 
-(define (class-slot-set-supers-union! class slot base-class init)
+(define (class-slot-set-supers-union! class slot init)
   (class-slot-set! class slot
                    (apply append
                           (cons
@@ -228,5 +232,8 @@
                            (map (lambda (c)
                                   (class-slot-ref c slot))
                                 (filter
-                                 (lambda (c) (not (eq? base-class c)))
+                                 (lambda (c)
+                                   (member slot
+                                           (map slot-definition-name
+                                                (class-slots c))))
                                  (class-direct-supers class)))))))
