@@ -24,31 +24,6 @@ long gw_test_out_args(int arg1, int *arg2, char **arg3)
   return arg1;
 }
 
-/*======================================================================*/
-/* Generics */
-
-int gw_test_generic__int(int n)
-{
-  return n * n;
-}
-
-char *gw_test_generic__str_int (const char *str, int n)
-{
-  int i;
-  int len = strlen (str);
-  char *result;
-
-  result = malloc (n * len + 1);
-  if (!result)
-    return NULL;
-  
-  for (i = 0; i < n; i++)
-    memcpy (result + (i * len), str, len);
-  result[n * len] = '\0';
-
-  return result;
-}
-
 /*======================================================================*/
 /* For test-gw-enumeration */
 
